#!/bin/sh
######## Installation script for HP-DMMP

rhel45dep=()
rhel46dep=()
rhel5dep=(kpartx)
rhel51dep=(kpartx)
sles93dep=()
sles94dep=()
sles10dep=()
sles101dep=()

sdir="./SRPMS/"
dir="./RPMS/"
pack="HPDMmultipath-tools-"
dmver="4.0.0"

rhel45arr=( [0]="RHEL4.5" [1]="2\.6\.9-55\.*" [2]=${rhel45dep[@]} )
rhel46arr=( [0]="RHEL4.6" [1]="2\.6\.9-67\.*" [2]=${rhel46dep[@]} )
rhel5arr=( [0]="RHEL5" [1]="2\.6\.18-8\.\(1\.1[0-9]+*\|1\.[2-9]+*\|[2-9]+*\)" [2]=${rhel5dep[@]} )
rhel51arr=( [0]="RHEL5.1" [1]="2\.6\.18-53\.*" [2]=${rhel51dep[@]} )

rhelarr=( [0]=${rhel45arr[@]} [1]=${rhel5arr[@]} [2]=${rhel46arr[@]} [3]=${rhel51arr[@]} )

sles93arr=( [0]="SLES9.3" [1]="2\.6\.5-7\.\(28[3-9]\|29[0-9]\|[3-9][0-9][0-9]*\).*" [2]=${sles93dep[@]} )
sles94arr=( [0]="SLES9.4" [1]="2\.6\.5-7\.308.*" [2]=${sles94dep[@]} )
sles10arr=( [0]="SLES10" [1]="2\.6\.16\.21-0\.8.*" [2]=${sles10dep[@]} )
sles101arr=( [0]="SLES10.1" [1]="2\.6\.16\.46-0\.12.*" [2]=${sles101dep[@]} )

slesarr=( [0]=${sles93arr[@]} [1]=${sles10arr[@]} [2]=${sles101arr[@]} [3]=${sles94arr[@]} )

currentrel=( )
sles93kern="2\.6\.5-7\.\(28[3-9]\|29[0-9]\|[3-9][0-9][0-9]*\).*"
cfile="/etc/multipath.conf"
csavefile="/etc/multipath.conf.prevconfsave"

log="HPDMMP_Install.log"

## print_n_log routine -- argument 1 for display in screen and logging
##			  	   2 for logging only

print_n_log() {

	case $1 in
	1)
		echo -e "$2"
		echo -e "$2" >> $log
		;;
	2)
		echo -e "$2" >> $log
		;;
	esac

}

## is_present routine -- argument path

is_present() {
        if [ ! -e $1 ]; then
	        print_n_log 1 "This RPM could not be located. Please contact HP support.\nThis installation will be aborted."
		exit 2
        fi
}

## Uninstallation routine -- argument 1 for calling from installation  
##			     argument 2 for calling from uninstallation
## 	uninstall_return denotes the exit value

uninstall() {

uninstall_return=0
print_n_log 2 "Entering the uninstallation routine : $1"
print_n_log 1 "\nChecking for previous installation. Please wait...\n"
dmmpts=`( rpm -qa | grep multipath )`
if [ -n "$dmmpts" ]; then

   for dmmpt in $dmmpts; do

### Checking whether the rpm is device-mapper multipath rpm

     print_n_log 2 "Checking if the rpm $dmmpt is device-mapper multipath rpm..."
     if [ `rpm -qi $dmmpt | grep -c "device-mapper"` -ne 0 ]; then

	while :
	do
	echo -e "\rDo you wish to uninstall $dmmpt ?(y/n) : \c"
	read -n 1 ch
	if [ "$ch" = "y" ]; then

### Check if the rpm has dependency conflicts

		print_n_log 2 "Uninstalling.."
		print_n_log 1 "\n\nChecking for dependencies... \c"
                test_op=`rpm -e --test $dmmpt 2>> $log`
                if [ $? -eq 0 ]; then
                        print_n_log 1 "OK"
                else
			print_n_log 1 "\n$test_op\n"
			while :
			do
                        echo -e "\rDo you still wish to uninstall $dmmpt ?(y/n):\c"
                        read -n 1 ch
                        if [ "$ch" == "y" ]; then
				break
			elif [ "$ch" == "n" ]; then
				uninstall_return=1
				echo ""
                                return
			else
				sleep .2
				echo -e "\b \c"
                        fi
			done
                fi

### If called from installation, back-up the configuration file

		if [ $1 -eq 1 ]; then
                	if [ -f $cfile ]; then
                		`cp -f $cfile $csavefile 2>/dev/null`
				print_n_log 2 "Copied $cfile to $csavefile"
                	fi
		fi

### If the multipath daemon is running, stop it

		status=`/etc/init.d/multipathd status|grep -c "running"`
		if [ $status -eq 1 ]; then
			print_n_log 2 "Stopping multipathd daemon"
			`/etc/init.d/multipathd stop >> $log 2>> $log`
		fi

### un-install

		`rpm -e --nodeps $dmmpt >> $log 2>> $log`
		if [ $? -eq 0 ]; then
			print_n_log 1 "\n$dmmpt is Uninstalled successfully"
		else
			print_n_log 1 "\nUninstallation was not successful. Please refer to '$log' for more details."
			uninstall_return=2
		fi
		break

	elif [ "$ch" = "n" ]; then
	        uninstall_return=1
		echo ""
		break
	else
		sleep .2
		echo -e "\b \c"
	fi
	done
     fi
   done
else

### If called from un-installation, inform user that no package found.

	if [ $1 -eq 2 ]; then
		print_n_log 1 "\nDevice Mapper Multipath is not found in the system."
	fi
fi

}

## conf_multipath_daemon routine - to configure the daemon to start at boot time

conf_multipath_daemon() {
	flag=0
	echo -en "\n\nConfiguring multipath daemon to start at boot time.... \t"

	if [ `echo ${currentrel[0]}|grep -c "RHEL"` -eq 1 ]; then

		print_n_log 2 "\nConfiguring multipath daemon to start at boot time for RHEL"

		print_n_log 2 "\nChecking the configuration.."
		`chkconfig --list multipathd >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=1
		fi
		`chkconfig --level 35 multipathd on >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=1
		fi
		print_n_log 2 "\nChecking the configuration after change.."
		`chkconfig --list multipathd >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=1
		fi
		`multipath -v0 >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=1
		fi

	else

		print_n_log 2 "\nConfiguring multipath daemon to start at boot time for SLES"

		`insserv boot.multipath multipathd >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=1
		fi
		`chkconfig multipathd on >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=1
		fi
		`chkconfig boot.multipath on >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=2
		fi
		`/etc/init.d/boot.multipath start >> $log 2>> $log`
		if [ $? -ne 0 ]; then
			flag=2
		fi
	
	fi
	
	`/etc/init.d/multipathd start >> $log 2>> $log`
	if [ $? -ne 0 ]; then
		flag=1
	fi

	case $flag in
	0)	echo "OK"
		;;
	1)	echo -e "Failed\n"
                echo -e "Warning: Failed to enable multpathd service. Please refer to '$log' for more information."
                print_n_log 2 "\nWarning: Failed to enable multpathd service.\n"
		;;
	2)	echo -e "Failed\n"
                echo -e "Warning: Failed to enable boot.multipath service. Please refer to '$log' for more information.\n"
                print_n_log 2 "\nWarning: Failed to enable boot.multipath service.\n"	
		;;
	esac
	
}

## restore_conf_file routine - to restore the configuration file

restore_conf_file() {
	if [ -f $csavefile ]; then
		print_n_log 1 "\n\nWarning: Restoring your previous configuration, you will have to \n\t manually edit the configuration file for\n\t HP recommended parameters.\n\t Please refer user documentation for more details.\n\n"
		while :
		do
		echo -e "\rDo you wish to restore previous configuration? (y/n) : \c"
		read -n 1 ch
		if [ "$ch" = "y" ]; then
			print_n_log 2 "\nRestoring previous configuration..."
			`mv -f $csavefile $cfile 2>/dev/null` 
			print_n_log 2 "Moved $csavefile to $cfile"
			break
		elif [ "$ch" = "n" ]; then
			break
		else
			sleep .2
			echo -e "\b \c"
                fi
		done
	fi
}
##### Starts here ##########3

print_n_log 2 "\n\n******** HPDMMP Installation. Date : `date` **********\n" 

while :
do
clear

echo -e "\nHP Device Mapper MultiPath Ver${dmver} - Installation Menu\n"
echo -e "\t1. Install HP Device Mapper MultiPath"
echo -e "\t2. Uninstall Multipath Utilities"
echo -e "\t3. Exit\n"
echo -n "Enter choice [1/2/3] : "
read -n 1 choice

print_n_log 2 "Choice Entered : $choice" 
case $choice in
1)

###  check for hardware platform 

	print_n_log 2 "\nChecking for hardware platform...."

	hdw=`uname -i 2>/dev/null`
	print_n_log 2 "\tPlatform : $hdw "

	if [ "$hdw" = "i386" ]; then
		plat=i386
	elif [ "$hdw" = "ia64" -o "$hdw" = "GenuineIntel" ]; then
		plat=ia64
	elif [ "$hdw" = "x86_64" -o "$hdw" = "AuthenticAMD" ]; then
		plat=x86_64
	else
		print_n_log 1 "\nThis platform is not supported. Please refer the IRG for the supported configuration.\nThis installation will be aborted."
		exit 1
	fi

###  Check for OS

	print_n_log 2 "\nChecking for OS...."

##  For RHEL - use 'uname -r' to find the kernel version 

	if [ `grep -c "Red Hat" /proc/version` -ne 0 ]; then
	
		rpm_dir="/usr/src/redhat/RPMS/"
		osrel=`uname -r 2>/dev/null`
		print_n_log 2 "\tRed Hat Release - $osrel"
		for (( i = 0 ; i < ${#rhelarr[@]} ; i++ )) ;do
			tmp=( ${rhelarr[$i]} )
			if [ `echo $osrel | grep -c ${tmp[1]}` -eq 1 ]; then
				currentrel=( ${rhelarr[$i]} )
				break
			fi
		done
		if [ ${#currentrel[@]} -eq 0 ]; then
			print_n_log 1 "\nThis version of Red Hat is not supported.\nPlease refer the IRG for the supported configuration.\nThis installation will be aborted."
			exit 1
		fi

## For SLES - if platform is i386 make it i586 and use SPident to find if OS 
## is properly installed.

	elif [ `grep -c -i "SUSE" /proc/version` -ne 0 ]; then
	   
	   rpm_dir="/usr/src/packages/RPMS/"
	   if [ "$plat" = "i386" ]; then
		plat=i586
	   fi	
	   osrel=`uname -r 2>/dev/null`
           print_n_log 2 "\tSUSE release : $osrel."
    	   if [ `rpm -qa|grep -c SPident` -ne 0 ]; then

		spident_op=`SPident 2>/dev/null`
		print_n_log 2 "\tUsing SPident - $spident_op"
		if [ `echo ${spident_op}| grep -c NOT` -ne 0 ]; then
			print_n_log 1 "\nSystem is not up-to-date. Please run 'SPident' for more details.\nThis installation will be aborted."
			exit 1
		elif [ `echo ${spident_op}| grep found | grep SLES-10 | grep -c -v SP` -ne 0 ]; then
			currentrel=( ${slesarr[1]} )
		elif [ `echo ${spident_op}| grep found | grep -c SLE-10.*SP1` -ne 0 ]; then
			currentrel=( ${slesarr[2]} )
		elif [ `echo ${spident_op}| grep found | grep -c SLES-9.*SP3` -ne 0 -a `echo $osrel | grep -c $sles93kern` -eq 1 ]; then
			currentrel=( ${slesarr[0]} )
		elif [ `echo ${spident_op}| grep found | grep -c SLES-9.*SP4` -ne 0 ]; then
			currentrel=( ${slesarr[3]} )
		else
			print_n_log 1 "\nThis version of SUSE is not supported.\nPlease refer the IRG for the supported configuration.\nThis installation will be aborted."
			exit 1
		fi
	
    	   else

## If SPident is not installed in the system, use 'uname -r' to find the kernel version.

		echo -e "\nWarning : Could not verify the version of the OS"
		print_n_log 2 "\tSPident is not installed."
		for (( i = 0 ; i < ${#slesarr[@]} ; i++ )) ;do
			tmp=( ${slesarr[$i]} )
			if [ `echo $osrel | grep -c ${tmp[1]}` -eq 1 ]; then
				currentrel=( ${slesarr[$i]} )
				break
			fi
		done
		if [ ${#currentrel[@]} -eq 0 ]; then
			print_n_log 1 "\nThis version of SUSE is not supported.\nPlease refer the IRG for the supported configuration.\nThis installation will be aborted."
			exit 1
		fi
	   fi		
	else

## If neither RHEL nor SLES

		print_n_log 1 "\nThis Operating System is not supported.\nPlease refer the IRG for the supported configuration.\nThis installation will be aborted."
		exit 1
	fi

### Check dependencies

	print_n_log 2 "\nDependency Check..."
	deplist=( ${currentrel[2]} )
	depfail=0
	for dep in ${deplist[@]} ;do
		print_n_log 1 "\nChecking for dependency $dep ....  \c"
		pac=`rpm -qa|grep -c $dep`
		if [ $pac -eq 0 ]; then
			print_n_log 1 "not installed.\nInstall $dep and then restart HP DMMP tools installation"
			depfail=1
		else
			print_n_log 1 "OK"
		fi
	done
	if [ $depfail -eq 1 ]; then
		print_n_log 1 "\nOne or more dependent packages are not installed.\nThis installation will be aborted."
		exit 2
	fi	

### Copyright info #######
	
	clear
	echo -e "\nPlease read the License Agreement and hit Space to continue."
	more COPYING

### Check for the SRPM ####

	srcpath="${sdir}${pack}${dmver}-${currentrel[0]}.src.rpm"
	print_n_log 2 "\nSRPM\t: $srcpath"
	is_present $srcpath

### Validate SRPM for Build dependencies

	print_n_log 1 "\nChecking for Build dependencies... \n"
	build_test=`rpmbuild --rebuild --nobuild $srcpath 2>&1 |grep -e "dependencies" -e "needed"`
	if [ $? -eq 0 ]; then
		print_n_log 1 "$build_test"
		print_n_log 1 "\nThis installation will be aborted."
		exit 2
        fi
		
### Create RPM and mv to RPMS directory ###

	print_n_log 1 "\nBuilding ${pack}${dmver} .... \c"
	`rpmbuild --rebuild $srcpath >> $log 2>>$log`
	if [ $? -ne 0 ]; then
        	print_n_log 1 "\n\nRPM Build was not successful. Please refer to '$log' for more information.\nThis installation will be aborted."
		exit 2
        fi

	rpm_path="${rpm_dir}/${plat}/${pack}${dmver}-${currentrel[0]}.${plat}.rpm"	
	print_n_log 2 "\nRPM Build path\t: $rpm_path"
	is_present $rpm_path

	if [ ! -d $dir ]; then
		`mkdir $dir >>$log 2>>$log`
	fi
	`mv -f $rpm_path $dir >> $log 2>> $log`
	
### Check for the SRPM ####

	path="${dir}${pack}${dmver}-${currentrel[0]}.${plat}.rpm"
	print_n_log 2 "\nRPM\t: $path"
	is_present $path

###check if already installed 

	print_n_log 2 "\nCheck if already installed..."
	uninstall 1
	if [ $uninstall_return -ne 0 ]; then
		print_n_log 1 "\nThis installation will be aborted."
		exit $uninstall_return
	fi

### Install HPDMMP tools

	echo ""
	while :
	do
	echo -e "\rDo you wish to install ${pack}${dmver}? (y/n) : \c"
	read -n 1 ch
	if [ "$ch" = "y" ]; then
		print_n_log 2 "\nInstalling HPDMMP..."
		`rpm -ivh $path >> $log 2>> $log`
		if [ $? -ne 0 ]; then
                        print_n_log 1 "\nInstallation was not successful. Please refer to '$log' for more information."
			exit 2
                fi
		break
	elif [ "$ch" = "n" ]; then
		echo ""
		exit 1
	else
		sleep .2
		echo -e "\b \c"
	fi
	done

### Restore configuration
	restore_conf_file

### Configuring multipath daemon to start at boot time
	conf_multipath_daemon
	print_n_log 1 "\n${pack}${dmver} is installed successfully."
	
	exit 0
	;;
2)
	uninstall 2
	
	#Changes for QuiX Issue -1000754191

	echo ""
	echo -n "Press any Key to Continue.."
	read -n 1 anykey
	#exit 0
	;;
3)
	echo ""
	exit 0
	;;
*)
	print_n_log 1 "\nInvalid option"
	`sleep .5`
	;;
esac
done
